@@ -1,4 +1,4 @@
-// Copyright (c) 2014-2020, The Monero Project
+// Copyright (c) 2014-2022, The Monero Project
 // 
 // All rights reserved.
 // 
@@ -31,6 +31,7 @@
 #pragma once
 
 #include <iosfwd>
+#include <iterator>
 #include <list>
 #include <string>
 #include <vector>
@@ -46,7 +47,6 @@
 #include "crypto/crypto.h"
 #include "cryptonote_config.h"
 #include "net/enums.h"
-#include "net/local_ip.h"
 #include "p2p_protocol_defs.h"
 #include "syncobj.h"
 
@@ -109,8 +109,8 @@ namespace nodetool
     bool get_white_peer_by_index(peerlist_entry& p, size_t i);
     bool get_gray_peer_by_index(peerlist_entry& p, size_t i);
     template<typename F> bool foreach(bool white, const F &f);
-    void evict_host_from_white_peerlist(const peerlist_entry& pr);
-    bool append_with_peer_white(const peerlist_entry& pr);
+    void evict_host_from_peerlist(bool white, const peerlist_entry& pr);
+    bool append_with_peer_white(const peerlist_entry& pr, bool trust_last_seen = false);
     bool append_with_peer_gray(const peerlist_entry& pr);
     bool append_with_peer_anchor(const anchor_peerlist_entry& ple);
     bool set_peer_just_seen(peerid_type peer, const epee::net_utils::network_address& addr, uint32_t pruning_seed, uint16_t rpc_port, uint32_t rpc_credits_per_hash);
@@ -120,6 +120,7 @@ namespace nodetool
     bool get_and_empty_anchor_peerlist(std::vector<anchor_peerlist_entry>& apl);
     bool remove_from_peer_anchor(const epee::net_utils::network_address& addr);
     bool remove_from_peer_white(const peerlist_entry& pe);
+    template<typename F> size_t filter(bool white, const F &f); // f returns true: drop, false: keep
     
   private:
     struct by_time{};
@@ -183,6 +184,7 @@ namespace nodetool
   private: 
     void trim_white_peerlist();
     void trim_gray_peerlist();
+    static peerlist_entry get_nth_latest_peer(peers_indexed& peerlist, size_t n);
 
     friend class boost::serialization::access;
     epee::critical_section m_peerlist_lock;
@@ -213,6 +215,16 @@ namespace nodetool
     }
   }
   //--------------------------------------------------------------------------------------------------
+  inline
+  peerlist_entry peerlist_manager::get_nth_latest_peer(peers_indexed& peerlist, const size_t n)
+  {
+    // Is not thread-safe nor does it check bounds. Do this before calling. Indexing starts at 0.
+    peers_indexed::index<by_time>::type& by_time_index = peerlist.get<by_time>();
+    auto by_time_it = --by_time_index.end();
+    std::advance(by_time_it, -static_cast<long long>(n));
+    return *by_time_it;
+  }
+  //--------------------------------------------------------------------------------------------------
   inline 
   bool peerlist_manager::merge_peerlist(const std::vector<peerlist_entry>& outer_bs, const std::function<bool(const peerlist_entry&)> &f)
   {
@@ -234,8 +246,7 @@ namespace nodetool
     if(i >= m_peers_white.size())
       return false;
 
-    peers_indexed::index<by_time>::type& by_time_index = m_peers_white.get<by_time>();
-    p = *epee::misc_utils::move_it_backward(--by_time_index.end(), i);    
+    p = peerlist_manager::get_nth_latest_peer(m_peers_white, i);
     return true;
   }
   //--------------------------------------------------------------------------------------------------
@@ -246,8 +257,7 @@ namespace nodetool
     if(i >= m_peers_gray.size())
       return false;
 
-    peers_indexed::index<by_time>::type& by_time_index = m_peers_gray.get<by_time>();
-    p = *epee::misc_utils::move_it_backward(--by_time_index.end(), i);    
+    p = peerlist_manager::get_nth_latest_peer(m_peers_gray, i);
     return true;
   }
   //--------------------------------------------------------------------------------------------------
@@ -329,12 +339,12 @@ namespace nodetool
     ple.pruning_seed = pruning_seed;
     ple.rpc_port = rpc_port;
     ple.rpc_credits_per_hash = rpc_credits_per_hash;
-    return append_with_peer_white(ple);
+    return append_with_peer_white(ple, true);
     CATCH_ENTRY_L0("peerlist_manager::set_peer_just_seen()", false);
   }
   //--------------------------------------------------------------------------------------------------
   inline
-  bool peerlist_manager::append_with_peer_white(const peerlist_entry& ple)
+  bool peerlist_manager::append_with_peer_white(const peerlist_entry& ple, bool trust_last_seen)
   {
     TRY_ENTRY();
     if(!is_host_allowed(ple.adr))
@@ -346,7 +356,7 @@ namespace nodetool
     if(by_addr_it_wt == m_peers_white.get<by_addr>().end())
     {
       //put new record into white list
-      evict_host_from_white_peerlist(ple);
+      evict_host_from_peerlist(true, ple);
       m_peers_white.insert(ple);
       trim_white_peerlist();
     }else
@@ -357,7 +367,8 @@ namespace nodetool
         new_ple.pruning_seed = by_addr_it_wt->pruning_seed;
       if (by_addr_it_wt->rpc_port && ple.rpc_port == 0) // guard against older nodes not passing RPC port around
         new_ple.rpc_port = by_addr_it_wt->rpc_port;
-      new_ple.last_seen = by_addr_it_wt->last_seen; // do not overwrite the last seen timestamp, incoming peer list are untrusted
+      if (!trust_last_seen)
+        new_ple.last_seen = by_addr_it_wt->last_seen; // do not overwrite the last seen timestamp, incoming peer lists are untrusted
       m_peers_white.replace(by_addr_it_wt, new_ple);
     }
     //remove from gray list, if need
@@ -435,9 +446,7 @@ namespace nodetool
     }
 
     size_t random_index = crypto::rand_idx(m_peers_gray.size());
-
-    peers_indexed::index<by_time>::type& by_time_index = m_peers_gray.get<by_time>();
-    pe = *epee::misc_utils::move_it_backward(--by_time_index.end(), random_index);
+    pe = peerlist_manager::get_nth_latest_peer(m_peers_gray, random_index);
 
     return true;
 
@@ -519,5 +528,26 @@ namespace nodetool
     CATCH_ENTRY_L0("peerlist_manager::remove_from_peer_anchor()", false);
   }
   //--------------------------------------------------------------------------------------------------
+  template<typename F> size_t peerlist_manager::filter(bool white, const F &f)
+  {
+    size_t filtered = 0;
+    TRY_ENTRY();
+    CRITICAL_REGION_LOCAL(m_peerlist_lock);
+    peers_indexed::index<by_addr>::type& sorted_index = white ? m_peers_gray.get<by_addr>() : m_peers_white.get<by_addr>();
+    auto i = sorted_index.begin();
+    while (i != sorted_index.end())
+    {
+      if (f(*i))
+      {
+        i = sorted_index.erase(i);
+        ++filtered;
+      }
+      else
+        ++i;
+    }
+    CATCH_ENTRY_L0("peerlist_manager::filter()", filtered);
+    return filtered;
+  }
+  //--------------------------------------------------------------------------------------------------
 }
 
