@@ -333,6 +333,19 @@ class Wallet(object):
         }
         return self.rpc.send_json_rpc_request(generate_from_keys)
 
+    def set_subaddress_lookahead(self, major_idx: int, minor_idx: int, password = ""):
+        lookahead = {
+            'method': 'set_subaddress_lookahead',
+            'jsonrpc': '2.0',
+            'params' : {
+                'password': password,
+                'major_idx': major_idx,
+                'minor_idx': minor_idx
+            },
+            'id': '0'
+        }
+        return self.rpc.send_json_rpc_request(lookahead)
+
     def open_wallet(self, filename, password='', autosave_current = True):
         open_wallet = {
             'method': 'open_wallet',
